import resolve from '@rollup/plugin-node-resolve';
import commonjs from '@rollup/plugin-commonjs';
import json from '@rollup/plugin-json';
import path from 'path';
import esbuild from 'rollup-plugin-esbuild';
import alias from '@rollup/plugin-alias';
import replace from '@rollup/plugin-replace';
import dts from 'rollup-plugin-dts';
// @ts-ignore
import pkgs from './package.json' with {type: 'json'};

const isDev = process.env.NODE_ENV === 'development';
const input = './src/index.ts';
/**
 * @type {import('rollup').RollupOptions}
 */
const config = {
  input,
  output: {
    dir: './dist',
    entryFileNames: 'assistant-module.mjs',
    chunkFileNames: '[name]-[hash].mjs',
    format: 'esm',
  },
  plugins: [
    replace({
      preventAssignment: true, // 防止意外赋值
      DEV_SERVER: JSON.stringify(isDev), // 替换 process.env.NODE_ENV
      VERSION: JSON.stringify(pkgs.version),
    }),
    alias({
      // only esbuild needs to be configured
      entries: [
        { find: '@', replacement: path.resolve('src') }, // 配置 @ 为 src 目录
        { find: 'http', replacement: 'node:http' },
        { find: 'https', replacement: 'node:https' },
        { find: 'fs', replacement: 'node:fs' },
        { find: 'path', replacement: 'node:path' },
        { find: 'crypto', replacement: 'node:crypto' },
        { find: 'zlib', replacement: 'node:zlib' },
        { find: 'stream', replacement: 'node:stream' },
        { find: 'net', replacement: 'node:net' },
        { find: 'tty', replacement: 'node:tty' },
        { find: 'tls', replacement: 'node:tls' },
        { find: 'buffer', replacement: 'node:buffer' },
        { find: 'timers', replacement: 'node:timers' },
        // { find: 'string_decoder', replacement: 'node:string_decoder' },
        { find: 'dns', replacement: 'node:dns' },
        { find: 'domain', replacement: 'node:domain' },
        { find: 'os', replacement: 'node:os' },
        { find: 'events', replacement: 'node:events' },
        { find: 'url', replacement: 'node:url' },
        { find: 'assert', replacement: 'node:assert' },
        { find: 'util', replacement: 'node:util' },
      ],
    }),
    resolve({
      preferBuiltins: true, // 强制优先使用内置模块
    }),
    commonjs(),
    esbuild({
      target: 'node22', //
      minify: false, // 启用代码压缩
      tsconfig: 'tsconfig.json',
    }),
    json(),
  ],
  external: [
    /@kevisual\/router(\/.*)?/, //, // 路由
    /@kevisual\/use-config(\/.*)?/, //
  ],
};
const dtsConfig = [{
  input,
  output: {
    file: 'dist/assistant-module.d.ts',
    format: 'esm',
  },
  plugins: [dts()],
}];

const moduleConfig = {
  input: './src/assistant-proxy.ts',
  output: {
    file: 'dist/assistant-proxy.mjs',
    format: 'esm',
  },
  plugins: [
    alias({
      entries: [{ find: '@', replacement: path.resolve('src') }],
    }),
    resolve({
      preferBuiltins: true, // 强制优先使用内置模块
    }),
    commonjs(),
    esbuild({
      target: 'node22', //
    }),
    json(),
  ],

};
const moduleDtsConfig = [{
  input: './src/assistant-proxy.ts',
  output: {
    file: 'dist/assistant-proxy.d.ts',
    format: 'esm',
  },
  plugins: [dts()],
}];

const assistantConfigConfig = {
  input: './src/assistant-config.ts',
  output: {
    file: 'dist/assistant-config.mjs',
    format: 'esm',
  },
  plugins: [
    alias({
      entries: [{ find: '@', replacement: path.resolve('src') }],
    }),
    resolve({
      preferBuiltins: true, // 强制优先使用内置模块
    }),
    commonjs(),
    esbuild({
      target: 'node22', //
    }),
    json(),
  ],
};
const assistantConfigDtsConfig = [{
  input: './src/assistant-config.ts',
  output: {
    file: 'dist/assistant-config.d.ts',
    format: 'esm',
  },
  plugins: [dts()],
}];

const assistantProcessConfig = {
  input: './src/assistant-process.ts',
  output: {
    file: 'dist/assistant-process.mjs',
    format: 'esm',
  },
  plugins: [
    alias({
      entries: [{ find: '@', replacement: path.resolve('src') }],
    }),
    resolve({
      preferBuiltins: true, // 强制优先使用内置模块
    }),
    commonjs(),
    esbuild({
      target: 'node22', //
    }),
    json(),
  ],
};
const assistantProcessDtsConfig = [{  
  input: './src/assistant-process.ts',
  output: {
    file: 'dist/assistant-process.d.ts',
    format: 'esm',
  },
  plugins: [dts()],
}];



export default [config, ...dtsConfig, moduleConfig, ...moduleDtsConfig, assistantConfigConfig, ...assistantConfigDtsConfig, assistantProcessConfig, ...assistantProcessDtsConfig];